<pre class='metadata'>
Title: Input Modality
Status: w3c/ED
Work Status: Exploring
ED: http://github.com/WICG/modality/modality.html
Shortname: modality
Level: 1
Editor: Alice Boxhall, Google, aboxhall@google.com
Abstract: This specification describes an extension to the :focus pseudoclass that allows developers to change focus behavior based on the input modality of the user interaction that led to the focus occurring.
Repository: WICG/modality
</pre>

Introduction {#introduction}
============================

Focus rings highlight the current focus location. Particularly during keyboard
interactions, focus rings provide important feedback to the user by showing the
current interaction point. This is particularly important for accessibility use
cases where non-keyboard interactions are unavailable.

User agents employ a set of heuristics to decide when to show a focus ring.
For example, typically, selecting an element via the keyboard results in a
focus ring being displayed, while selecting an element via a mouse interaction
does not.

Unfortunately, focus rings are not easily stylable. While the :focus pseudo-class
has been available for some time, this matches every focussed element rather than
just those which the user agent chooses to highlight.

This specification introduces a new pseudo-class, '':focusring'', that matches an
element only when the user agent would choose to show a focus ring for that
element.

<div class='example'>

To change the focus ring style to a dotted red outline, without changing the
behavior of when a ring is shown:

<pre lang='css'>
:focusring {
  outline: 1px dotted red;
}
</pre>

</div>

The '':focusring'' pseudo-class {#the-focusring-pseudo-class}
=============================================================

The focusring pseudo-class, <dfn id='focusring-pseudo'>:focusring</dfn>,
is a dynamic user action pseudo-class.

For native interaction elements, :focusring applies when the user agent would
choose to display a focusring in the absence of UA or author stylesheets.

For other elements, :focusring applies according to a heuristic that may
match the behavior of the UA for native button elements. An example heuristic
is included non-normatively in the appendix.

<div class='example'>
For example, most UAs choose to display focusrings on native text fields
whenever they are focussed, and on native buttons only when selected via
the keyboard.
</div>

<div class='note'>
Authors who build non-native interactive elements that are text-field like
in nature should consider adding a '':focus'' rule to that element to
ensure a focusring is displayed whenever focussed, rather than only when
the heuristic matches.
</div>

Appendix A: Example Heuristic for non-native elements {#appendix-a}
===================================================================

This appendix is non-normative.

A user agent might choose to use the following heuristic for deciding when
to apply '':focusring'' to elements that are not native interaction elements:

When focussing an element:
*   if the last interaction was a keyboard event, and the interaction
    was less than 100ms in the past, apply '':focusring''.

When unfocussing an element:
*   stop applying '':focusring''.

When handling a keyboard event:
*   if a non-native element is focussed, apply '':focusring'' to that element.
